//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-imaging-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using GrapeCity.Documents.Drawing;
using GrapeCity.Documents.Imaging;
using GrapeCity.Documents.Svg;
using GrapeCity.Documents.Html;
using Color = System.Drawing.Color;
using Font = GrapeCity.Documents.Text.Font;
using RectangleF = System.Drawing.RectangleF;
using SizeF = System.Drawing.SizeF;

namespace DsImagingWeb.Demos.Common
{
    public static class Util
    {
        private static int s_gen = 0;
        private static DateTime s_fixedTime = new DateTime(1989, 6, 21, 1, 23, 45, DateTimeKind.Utc);

        public static class MimeTypes
        {
            public const string JPEG = "image/jpeg";
            public const string PNG = "image/png";
            public const string TIFF = "image/tiff";
            public const string GIF = "image/gif";
            public const string BMP = "image/bmp";
            public const string ICO = "image/x-icon";
            public const string SVG = "image/svg+xml";
            public const string WEBP = "image/webp";
        }

        public static bool NoRandom { get; set; } = false;

        public static Random NewRandom()
        {
            if (NoRandom)
                return new Random(0);
            else
                return new Random((int)DateTime.Now.Ticks + Interlocked.Increment(ref s_gen));
        }

        public static DateTime TimeNow()
        {
            if (NoRandom)
                return s_fixedTime;
            else
                return DateTime.Now;
        }

        private static string s_browserBasePath = null;
        private static readonly object s_browserBasePath_lock = new object();

        // Set this to use the browser from a known location:
        public static void SetBrowserBasePath(string browserBasePath)
        {
            lock (s_browserBasePath_lock)
                s_browserBasePath = browserBasePath;
        }

        // DsHtml v6.0+: this method provides a single target location to be used
        // by the online samples, and also by all samples downloaded by a user,
        // so that multiple samples can share the same browser installation
        // rather than each sample downloading and installing its own copy.
        // On a system with Chrome installed, the BrowserFetcher.GetSystemChromePath()
        // call ensures that the installed browser is used.
        public static string GetBrowserPath()
        {
            if (s_browserBasePath == null)
            {
                var systemPath = BrowserFetcher.GetSystemChromePath();
                if (systemPath != null)
                    return systemPath;
            }
            var path = s_browserBasePath ?? Path.Combine(Path.GetTempPath(), ".gc-chromium");
            var bf = new BrowserFetcher() { DestinationFolder = path };
            return bf.GetDownloadedPath();
        }

        // DsHtml v6.0:
        public static GcHtmlBrowser NewHtmlBrowser()
        {
            var path = GetBrowserPath();
            try
            {
                if (RuntimeInformation.IsOSPlatform(OSPlatform.Linux))
                    return new GcHtmlBrowser(path, new LaunchOptions { RunWithNoSandbox = true });
                else
                    return new GcHtmlBrowser(path);
            }
            catch (Exception ex)
            {
                if (path == BrowserFetcher.GetSystemChromePath())
                    throw;
                else
                    throw new Exception("Exception occurred when launching the browser (see inner exception for details), " +
                        $"the browser installation in {Path.GetDirectoryName(path)} may be corrupt. Please remove it and try again.", ex);
            }
        }

        // https://stackoverflow.com/questions/4286487/is-there-any-lorem-ipsum-generator-in-c
        public static string LoremIpsum(
            int numParagraphs = 5,
            int minSentences = 5,
            int maxSentences = 10,
            int minWords = 5,
            int maxWords = 30)
        {
            var words = new[]
            {
                "lorem", "ipsum", "dolor", "sit", "amet", "consectetur", "adipiscing", "elit", "mauris", "id", "volutpat",
                "tellus", "ullamcorper", "sem", "praesent", "et", "ante", "laoreet", "lobortis", "nunc", "congue",
                "nisi", "donec", "ac", "tempus", "sed", "feugiat", "pulvinar", "pharetra", "turpis", "nonummy",
                "at", "felis", "eget", "molestie", "euismod", "non", "aliquet", "diam", "proin", "mi", "nibh",
                "massa", "tincidunt", "ut", "dolore", "magna", "aliquam", "erat",
            };
            int commaMax = 10;
            var rnd = NewRandom();
            int numSentences = rnd.Next(maxSentences - minSentences) + minSentences + 1;
            int numWords = rnd.Next(maxWords - minWords) + minWords + 1;

            StringBuilder result = new StringBuilder();
            for (int p = 0; p < numParagraphs; p++)
            {
                for (int s = 0; s < numSentences; s++)
                {
                    for (int w = 0; w < numWords; w++)
                    {
                        string word = words[rnd.Next(words.Length)];
                        if (w == 0)
                        {
                            result.Append(char.ToUpper(word[0]));
                            result.Append(word.Substring(1));
                        }
                        else
                        {
                            if (rnd.Next(commaMax) == 0)
                                result.Append(',');
                            result.Append(' ');
                            result.Append(word);
                        }
                    }
                    result.Append(". ");
                }
                result.Append("\r\n");
            }
            return result.ToString();
        }

        // https://stackoverflow.com/questions/273313/randomize-a-listt
        public static void Shuffle<T>(this IList<T> list)
        {
            var rnd = NewRandom();
            int n = list.Count;
            while (n > 1)
            {
                n--;
                int k = rnd.Next(n + 1);
                T value = list[k];
                list[k] = list[n];
                list[n] = value;
            }
        }

        /// <summary>
        /// Adds a note to the document - a text fragment on a yellow background.
        /// Used to insert meta info into sample documents.
        /// </summary>
        /// <param name="text">The text to draw.</param>
        /// <param name="g">The graphics to draw on.</param>
        /// <param name="bounds">The text bounds. If null, whole page with 1" margins all around is used.</param>
        /// <returns>The actual rectangle occupied by the note.</returns>
        public static RectangleF AddNote(string text, GcGraphics g, RectangleF? bounds = null)
        {
            var tl = g.CreateTextLayout();
            var pad = g.Resolution / 8f;
            // tl.DefaultFormat.Font = StandardFonts.Helvetica;
            tl.DefaultFormat.FontSize = 12;
            if (!bounds.HasValue)
            {
                float width, height;
                if (g is GcBitmapGraphics gBmp)
                {
                    width = gBmp.Width;
                    height = gBmp.Height;
                }
                else if (g is GcSvgGraphics gSvg)
                {
                    width = gSvg.Width;
                    height = gSvg.Height;
                }
                else
                    throw new Exception("Unknown GcGraphics implementation.");

                var margin = g.Resolution; // 1" margins by default
                bounds = new RectangleF(margin, margin, width - margin * 2, height - margin * 2);
            }
            tl.MaxWidth = bounds.Value.Width;
            tl.MaxHeight = bounds.Value.Height;
            tl.MarginAll = pad;
            tl.Append(text);
            tl.PerformLayout(true);

            var rect = new RectangleF(bounds.Value.X, bounds.Value.Y, bounds.Value.Width, tl.ContentHeight + pad * 2);
            g.FillRectangle(rect, Color.PaleTurquoise);
            g.DrawRectangle(rect, Color.DarkSlateBlue, 0.5f);
            g.DrawTextLayout(tl, bounds.Value.Location);

            return rect;
        }

        /// <summary>
        /// Loads an image from a disk file.
        /// <para>
        /// The only point of this method is to provide better performance on a Windows system,
        /// by using GrapeCity.Documents.Drawing.Windows.WicImage which works via the
        /// Windows Imaging Component framework. If WicImage is not supported, this method
        /// defaults to platform-independent Image.FromFile() method.
        /// </para>
        /// </summary>
        /// <param name="path">The image path.</param>
        /// <returns>The Image object.</returns>
        public static IImage ImageFromFile(string path)
        {
            if (GrapeCity.Documents.Imaging.Windows.GcWicBitmap.IsSupported)
                return new GrapeCity.Documents.Imaging.Windows.GcWicBitmap(path);
            return new GcBitmap(path);
        }
    }
}
